use rand::seq::SliceRandom;
use rand::RngCore;
use crate::guess::*;
use crate::constraints;

pub trait Guesser<const N: usize> {
    fn push(&mut self, _result: GuessResult<N>) {}
    fn guess(&mut self) -> Word<N>;
    fn get_possible_words(&self) -> Dictionary<N>;
}

pub struct RandomConstraintGuesser<'a, const N: usize> {
    possible_words: Dictionary<N>,
    history: GuessHistory<N>,
    constraints: &'a mut dyn constraints::Constraints<N>,
    rng: &'a mut dyn RngCore,
}

impl<'a, const N: usize> RandomConstraintGuesser<'a, N> {
    pub fn new(
        dictionary: &Dictionary<N>,
        constraints: &'a mut dyn constraints::Constraints<N>,
        rng: &'a mut dyn RngCore,
    ) -> Self {
        Self {
            possible_words: dictionary.to_vec(),
            history: GuessHistory::new(),
            constraints,
            rng,
        }
    }
}

impl<'a, const N: usize> Guesser<N> for RandomConstraintGuesser<'a, N> {
    fn guess(&mut self) -> Word<N> {
        *self.possible_words.choose(&mut self.rng).unwrap()
    }

    fn push(&mut self, result: GuessResult<N>) {
        self.history.push(result);
        self.constraints.push(result);
        let constraints = &self.constraints;
        self.possible_words.retain(|&word| constraints.check(word));
    }

    fn get_possible_words(&self) -> Dictionary<N> {
        return self.possible_words.to_vec();
    }
}
